package com.avaya.sce.pdc.json;

import com.avaya.sce.runtimecommon.ITraceInfo;
import com.avaya.sce.runtimecommon.IVariableField;
import com.avaya.sce.runtimecommon.LoggerUtil;
import com.avaya.sce.runtimecommon.SCESession;
import com.avaya.sce.runtimecommon.VariableName;

public class Utils {

	public static String getValueFromParameter(String varStr, String fieldStr, boolean isConstant, SCESession mySession) {
		String value = null;
		if ((varStr != null) && (varStr.equals("") == false)) {
			if (isConstant == true) {
				value = varStr;
			} else {
				if ((fieldStr != null) && (fieldStr.equals("") == true)) {
					fieldStr = null;
				}
				IVariableField field = mySession.getVariableField(new VariableName(varStr, fieldStr));
				value = field.getStringValue().trim();
			}
		}
		return value;
	}

	/**
	 * This method parses the message text and wherever a session variable is
	 * found replaces the tag with the value of the variable.
	 * 
	 * @param mySession
	 *            the session
	 * @param messageText
	 *            the message text
	 * @return the string with replaced variables.
	 */
	public static String substituteVaraiables(SCESession mySession, String value) {
		StringBuilder text = new StringBuilder(value);
		/* Reverse search the string buffer to replace the variable {var:field} with the value
		 * (work backwards so the string buffer can grow with out affecting the indexing).
		 */
		int end = text.lastIndexOf("}_");
		while (end > -1) {
			int start = text.lastIndexOf("_{", end);
			/*a backslash is used as an escape char.  If curly brace
			 *is preceded by a backslash, we treat the contents as is and do
			 * not replace with variable value.
			 */
			if (start > 0 && text.charAt(start - 1) == '\\') {
				/* remove the back slash and indicate no variable was found. */
				text.replace(start - 1, start, "");
				start = -1;
			}
			if (start == -1) {
				break; /* no variable start tag found. */
			}
			String var = text.substring(start + 2, end);
			LoggerUtil.writeTrace(mySession, ITraceInfo.TRACE_LEVEL_DEBUG,
					"Replacing variable [" + var + "] with its value in the text.");
			String varValue = getVarValue(mySession, var);
			if (varValue == null) {
				LoggerUtil.writeTrace(mySession, ITraceInfo.TRACE_LEVEL_WARN,
						"Unable to get variable [" + var + "] value.  Text may be incomplete.");
			} else {

				/* insert the value into the string buffer */
				text.replace(start, end + 2, varValue);
			}

			/* Advance to the next variable */
			end = text.lastIndexOf("}_", start - 1);
		}
		return (text.toString());
	}

	private static String getVarValue(SCESession mySession, String varName) {
		if (varName == null || varName.length() == 0) {
			return null;
		}
		IVariableField field = mySession.getVariableField(new VariableName(varName));
		if (field != null) {
			return (field.getStringValue());
		}
		return null;
	}

}
