package com.avaya.sce.pdc.json.operations;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.avaya.sce.pdc.json.Utils;
import com.avaya.sce.runtime.IPluggableExecutable;
import com.avaya.sce.runtimecommon.Debugging;
import com.avaya.sce.runtimecommon.ICollection;
import com.avaya.sce.runtimecommon.IComplexVariable;
import com.avaya.sce.runtimecommon.IRuntimeSession;
import com.avaya.sce.runtimecommon.ITraceInfo;
import com.avaya.sce.runtimecommon.IVariable;
import com.avaya.sce.runtimecommon.IVariableField;
import com.avaya.sce.runtimecommon.SCERuntimeException;
import com.avaya.sce.runtimecommon.SCESession;
import com.avaya.sce.runtimecommon.VariableName;

public class JSONBuilder extends Debugging implements IPluggableExecutable {

	private String destVar;
	private String destField;
	private String pathVar;
	private String pathField;
	private boolean pathIsConstant;
	private String valueVar;
	private String valueField;
	private boolean valueIsConstant;
	private boolean useArray;

	private final static String OP = "Build JSON";

	public JSONBuilder(String destVar, String destField, String pathVar, String pathField, boolean pathIsConstant, String valueVar,
			String valueField, boolean valueIsConstant, boolean useArray) {

		this.destVar = destVar;
		this.destField = destField;
		this.pathVar = pathVar;
		this.pathField = pathField;
		this.pathIsConstant = pathIsConstant;
		this.valueVar = valueVar;
		this.valueField = valueField;
		this.valueIsConstant = valueIsConstant;
		this.useArray = useArray;
	}

	public String getId() {
		return ("com.avaya.sce.pdc.json");
	}

	public String getTraceMessage() {
		return (OP + " ->" + destVar + ":" + destField);
	}

	private Object processSimple(String varStr, String fieldStr, SCESession mySession) {
		IVariable var = mySession.getVariable(varStr);
		if ((fieldStr != null) && (fieldStr.equals("") == true)) {
			fieldStr = null;
		}
		IVariableField field = mySession.getVariableField(new VariableName(varStr, fieldStr));
		if (var.isCollection() == true) {
			ICollection collection = var.getCollection();
			ArrayList<String> values = new ArrayList<String>();
			collection.mark();
			collection.reset();
			int count = collection.size();
			for (int index = 0; index < count; index++) {
				collection.next();
				values.add(field.getStringValue());
			}
			collection.resetToMark();
			return (values.toArray(new String[0]));
		} else {
			if (useArray == true) {
				String[] v = new String[1];
				v[0] = field.getStringValue().trim();
				return (v);
			} else {
				return (field.getStringValue().trim());
			}
		}
	}

	private Object processConstant(String varStr) {
		if (useArray == true) {
			String[] v = new String[1];
			v[0] = varStr;
			return (v);
		} else {
			return (varStr);
		}
	}

	private Object load(String varName, String fieldName, boolean isConstant, String segmentName, Object json, SCESession mySession,
			boolean replace) throws Exception {

		if (isConstant == true) {
			Object o = processConstant(varName);
			if (json instanceof String) {
				return (o);
			} else {
				JSONObject j = (JSONObject) json;
				if (o.getClass().isArray() == true) {
					o = new JSONArray(o);
				}
				j.put(segmentName, o);
			}
			return (null);
		}

		IVariable var = mySession.getVariable(varName);
		if (fieldName == null) {
			fieldName = "";
		}
		if ((var.isSimpleVariable() == true) || (fieldName.isEmpty() == false)) {
			Object o = processSimple(varName, fieldName, mySession);
			if (json instanceof String) {
				return (o);
			} else {
				JSONObject j = (JSONObject) json;
				if (o.getClass().isArray() == true) {
					o = new JSONArray(o);
				}
				j.put(segmentName, o);
			}
		} else {
			IComplexVariable cx = var.getComplexVariable();
			if (var.isCollection() == false) {
				JSONObject content;
				if (replace == false) {
					content = new JSONObject();
					JSONObject j = (JSONObject) json;
					j.put(segmentName, content);
				} else {
					content = (JSONObject) json;
				}
				String[] names = cx.getFieldNames();
				for (int i = 0; i < names.length; i++) {
					content.put(names[i], cx.getField(names[i]).getStringValue());
				}
			} else {
				JSONArray content = new JSONArray();
				JSONObject j = (JSONObject) json;
				j.put(segmentName, content);
				ICollection collection = var.getCollection();
				collection.mark();
				collection.reset();
				while (collection.hasMore() == true) {
					JSONObject values = new JSONObject();
					collection.next();
					String[] names = cx.getFieldNames();
					for (int i = 0; i < names.length; i++) {
						values.put(names[i], cx.getField(names[i]).getStringValue());
					}
					content.put(values);
				}
				collection.resetToMark();
			}
		}
		return (null);
	}

	public void execute(IRuntimeSession rtSession) {
		String resultMessage = "ERROR : ";
		SCESession mySession = (SCESession) rtSession;
		Exception ex = null;

		try {
			/* set fields to null if it they are empty */
			if ((destField != null) && (destField.equals("") == true)) {
				destField = null;
			}
			if ((pathField != null) && (pathField.equals("") == true)) {
				pathField = null;
			}
			if ((valueField != null) && (valueField.equals("") == true)) {
				valueField = null;
			}

			IVariableField dest = mySession.getVariableField(new VariableName(destVar, destField));
			String path = Utils.getValueFromParameter(pathVar, pathField, pathIsConstant, mySession);

			if (rtSession.isTraceEnabled() == true) {
				rtSession.getTraceOutput().writeln(ITraceInfo.TRACE_LEVEL_DEBUG, "Path : " + path);
			}
			if (path == null) {
				path = "/";
			}
			path = path.trim();
			path = Utils.substituteVaraiables(mySession, path);
			if (path.equalsIgnoreCase(".") == true) {
				path = "/";
			}
			if (path.length() == 0) {
				path = "/";
			}
			if (rtSession.isTraceEnabled() == true) {
				rtSession.getTraceOutput().writeln(ITraceInfo.TRACE_LEVEL_DEBUG, "True Path : " + path);
			}
			if ((path.length() > 1) && (path.startsWith("/") == true)) {
				path = path.substring(1);
			}

			if ((path.length() == 1) && (path.startsWith("/") == true)) {
				throw new SCERuntimeException( "Illegal path.  Path can be a name or /name.  / by it self is not allowed.");
			}

			/*
			 * Work our way into the JSON object and find the target. 
			 */
			String jsonStr = dest.getStringValue();
			Object json = null;
			try {
				json = new JSONObject(jsonStr);
			} catch (Exception e) {
				json = new JSONObject();
				if (mySession.isTraceEnabled()) {
					ITraceInfo tr = mySession.getTraceOutput();
					tr.writeln(ITraceInfo.TRACE_LEVEL_DEBUG, "Value was not a JSON string, initializing to empty");
				}
			}
			Object jsonMain = json;

			String[] segments = path.split("/");
			String segmentName = null;
			int index = -1;
			for (int i = 0; i < segments.length; i++) {
				segmentName = null;
				index = -1;
				int bi = segments[i].indexOf('[');
				int ei = segments[i].indexOf(']');
				if ((bi != -1) && (ei != -1)) {
					segmentName = segments[i].substring(0, bi);
					String temp = segments[i].substring(bi + 1, ei);
					index = Integer.parseInt(temp);
				} else {
					segmentName = segments[i];
				}
				boolean replace = false;
				if (i == (segments.length - 1)) {
					JSONArray array = null;
					if (index != -1) {
						JSONObject jobj = (JSONObject) json;
						array = jobj.getJSONArray(segmentName);
						if (index > array.length() - 1) {
							/* extent array if it is short of the index */
							for (int j = 0; j <= (index - array.length() + 1); j++) {
								array.put(new JSONObject());
							}
						}
						json = array.get(index);
						/* we are updating an array where complex variables are stored */
						replace = true;
					}
					Object o = load(valueVar, valueField, valueIsConstant, segmentName, json, mySession, replace);
					if ((o != null) && (replace == true)) {
						/* this happens when you are updating a simple array x[0], as opposed to an array of structures */
						array.put(index, o);
					}
				} else {
					if (index != -1) {
						/* array index encountered */
						JSONObject jobj = (JSONObject) json;
						JSONArray array = jobj.getJSONArray(segmentName);
						if (index > array.length() - 1) {
							/* extent array if it is short of the index */
							for (int j = 0; j <= (index - array.length() + 1); j++) {
								array.put(new JSONObject());
							}
						}
						json = array.get(index);
					} else {
						try {
							Object temp = null;
							JSONObject jobj = (JSONObject) json;
							temp = jobj.get(segmentName);
							json = temp;
						} catch (JSONException e) {
							JSONObject jobj = (JSONObject) json;
							JSONObject jNew = new JSONObject();
							jobj.put(segments[i], jNew);
							json = jNew;
						}
					}
				}
			}
			dest.setValue(jsonMain.toString());
		} catch (Exception e) {
			resultMessage += "Could not " + OP + " : " + e;
			ex = e;
			mySession.throwRTException(resultMessage, ex);
		}
	}
}
