package com.avaya.sce.pdc.json.operations;

import org.json.JSONArray;
import org.json.JSONObject;

import com.avaya.sce.pdc.json.Utils;
import com.avaya.sce.runtime.IPluggableExecutable;
import com.avaya.sce.runtimecommon.Debugging;
import com.avaya.sce.runtimecommon.ICollection;
import com.avaya.sce.runtimecommon.IComplexVariable;
import com.avaya.sce.runtimecommon.IRuntimeSession;
import com.avaya.sce.runtimecommon.ITraceInfo;
import com.avaya.sce.runtimecommon.IVariable;
import com.avaya.sce.runtimecommon.IVariableField;
import com.avaya.sce.runtimecommon.SCESession;
import com.avaya.sce.runtimecommon.VariableName;

public class JSONParser extends Debugging implements IPluggableExecutable {

	private String destVar;
	private String destField;
	private String srcVar;
	private String srcField;
	private String pathVar;
	private String pathField;
	private boolean pathIsConstant;

	private final static String OP = "Parse JSON";

	public JSONParser(String destVar, String destField, String srcVar, String srcField, String pathVar, String pathField,
			boolean pathIsConstant) {

		this.destVar = destVar;
		this.destField = destField;
		this.srcVar = srcVar;
		this.srcField = srcField;
		this.pathVar = pathVar;
		this.pathField = pathField;
		this.pathIsConstant = pathIsConstant;
	}

	public String getId() {
		return ("com.avaya.sce.pdc.json");
	}

	public String getTraceMessage() {
		return (OP + " ->" + destVar + ":" + destField);
	}

	private void store(String varName, String fieldName, Object json, SCESession mySession) throws Exception {
		IVariable var = mySession.getVariable(varName);
		if (var.isSimpleVariable() == true) {
			var.getSimpleVariable().setValue(json.toString());
		} else if ((fieldName != null) && (fieldName.isEmpty() == false)) {
			IComplexVariable cx = var.getComplexVariable();
			cx.getField(fieldName).setValue(json.toString());
		} else {
			IComplexVariable cx = var.getComplexVariable();
			JSONObject j = (JSONObject) json;
			String[] names = JSONObject.getNames(j);
			for (String name : names) {
				IVariableField field = cx.getField(name);
				if (field != null) {
					field.setValue(j.get(name));
				}
			}
		}
	}

	public void execute(IRuntimeSession rtSession) {
		String resultMessage = "ERROR : ";
		SCESession mySession = (SCESession) rtSession;
		Exception ex = null;

		try {
			/* set fields to null if it they are empty */
			if ((destField != null) && (destField.equals("") == true)) {
				destField = null;
			}
			if ((pathField != null) && (pathField.equals("") == true)) {
				pathField = null;
			}
			if ((srcField != null) && (srcField.equals("") == true)) {
				srcField = null;
			}

			IVariableField src = mySession.getVariableField(new VariableName(srcVar, srcField));
			String path = Utils.getValueFromParameter(pathVar, pathField, pathIsConstant, mySession);

			if (rtSession.isTraceEnabled() == true) {
				rtSession.getTraceOutput().writeln(ITraceInfo.TRACE_LEVEL_DEBUG, "Path : " + path);
			}
			if (path == null) {
				path = "/";
			}
			path = path.trim();
			path = Utils.substituteVaraiables(mySession, path);
			if (path.equalsIgnoreCase(".") == true) {
				path = "/";
			}
			if (path.length() == 0) {
				path = "/";
			}
			if (rtSession.isTraceEnabled() == true) {
				rtSession.getTraceOutput().writeln(ITraceInfo.TRACE_LEVEL_DEBUG, "True Path : " + path);
			}
			if ((path.length() > 1) && (path.startsWith("/") == true)) {
				path = path.substring(1);
			}

			/*
			 * Work our way into the JSON object and find the target. 
			 */
			String jsonStr = src.getStringValue();
			Object json = new JSONObject(jsonStr);
			String[] segments = path.split("/");
			for (int i = 0; i < segments.length; i++) {
				String segmentName = null;
				int index = -1;
				int bi = segments[i].indexOf('[');
				int ei = segments[i].indexOf(']');
				if ((bi != -1) && (ei != -1)) {
					segmentName = segments[i].substring(0, bi);
					String temp = segments[i].substring(bi + 1, ei);
					index = Integer.parseInt(temp);
				} else {
					segmentName = segments[i];
				}

				if (index != -1) {
					/* array index encountered */
					JSONObject jobj = (JSONObject) json;
					JSONArray array = jobj.getJSONArray(segmentName);
					json = array.get(index);
				} else {
					JSONObject jobj = (JSONObject) json;
					json = jobj.get(segmentName);
				}
			}

			/*
			 * Next store the target into and OD variable as either a complex or 
			 * just to string into a simple variable while also respecting array to collection.
			 */
			if (json instanceof JSONArray) {
				IVariable var = mySession.getVariable(destVar);
				ICollection collection = null;
				if (var.isCollection() == true) {
					var.removeCollection();
				}
				collection = var.addCollection();

				JSONArray j = (JSONArray) json;
				for (int i = 0; i < j.length(); i++) {
					Object o = j.get(i);
					store(destVar, destField, o, mySession);
					collection.append();
				}
				collection.reset();
				collection.next();
			} else {
				store(destVar, destField, json, mySession);
			}
		} catch (Exception e) {
			resultMessage += "Could not " + OP + " : " + e;
			ex = e;
			mySession.throwRTException(resultMessage, ex);
		}
	}
}
